﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.ComponentModel;
using System.Windows.Media.Animation;

namespace AnalogClockCustomControlLib
{
	public partial class AnalogClock : UserControl, INotifyPropertyChanged
	{
		private const double INITIAL_CLOCK_SIZE = 200;

		public AnalogClock()
		{
			InitializeComponent();
			this.Loaded += new RoutedEventHandler(AnalogClock_Loaded);
		}

		private void AnalogClock_Loaded(object sender, RoutedEventArgs e)
		{
			InitClock();
		}

		private void InitClock()
		{
			InitClockFace();

			if (System.ComponentModel.DesignerProperties.GetIsInDesignMode(this))
			{
				//SetTime(new DateTime(DateTime.Today.Ticks).AddSeconds(36010));
				SetTime(DateTime.Now);
				ClockAnimation.Stop(this);    // TODO: Doesn't work?
			}
			else
			{
				SetTime(DateTime.Now);
			}

			// Add 60 key frames to make the second hand advance in discrete steps
			for (int i = 1; i < 60; i++)
			{
				SecondHandAnimation_Frames.KeyFrames.Add(new DiscreteDoubleKeyFrame(i * 6, KeyTime.Uniform));
			}
		}

		private void InitClockFace()
		{
			const string DOT_TAG = "clockdot";
			const int TEXTBLOCK_SIZE = 20;
			const int DOT_COUNT = 12;
			const int DOT_SIZE = 4;
			const int DOT_MARGIN = 15;

			// First remove any existing minute markers ("dots")
			for (int i = gridClock.Children.Count - 1; i > 0; i--)
			{
				if (gridClock.Children[i] is FrameworkElement)
					if (((gridClock.Children[i] as FrameworkElement).Tag as string) == DOT_TAG)
						gridClock.Children.RemoveAt(i);
			}

			//double RADIUS_CLOCK = clockFace.ActualWidth / 2;
			double RADIUS_CLOCK = (INITIAL_CLOCK_SIZE / 2) - DOT_MARGIN;

			for (double i = 0; i < DOT_COUNT; i++)
			{
				double x = Math.Cos((i + 4) / DOT_COUNT * 2.0 * Math.PI) * RADIUS_CLOCK * 2.0;    // Use (i+4) in order to rotate numbers
				double y = Math.Sin((i + 4) / DOT_COUNT * 2.0 * Math.PI) * RADIUS_CLOCK * 2.0;

				if (UseNumbers)
				{
					// Add numbers, using TextBlocks. Unfortunately TextBlock doesn't offer vertical alignment of its 
					// text content. To get around this we wrap it in a grid.
					Grid grid = new Grid()
					{
						Tag = DOT_TAG,
						Width = TEXTBLOCK_SIZE,
						Height = TEXTBLOCK_SIZE,
						Margin = new Thickness(DOT_MARGIN - (TEXTBLOCK_SIZE / 2), DOT_MARGIN - (TEXTBLOCK_SIZE / 2),
							x + DOT_MARGIN - (TEXTBLOCK_SIZE / 2), y + DOT_MARGIN - (TEXTBLOCK_SIZE / 2)),
					};
					TextBlock text = new TextBlock()
					{
						Text = (i + 1).ToString(),
						FontSize = 12f,
						HorizontalAlignment = System.Windows.HorizontalAlignment.Center,
						VerticalAlignment = System.Windows.VerticalAlignment.Center,
					};
					grid.Children.Add(text);
					gridClock.Children.Add(grid);
				}
				else
				{
					// Add ellipses
					Ellipse dot = new Ellipse()
					{
						Tag = DOT_TAG,
						Width = DOT_SIZE,
						Height = DOT_SIZE,
						Margin = new Thickness(DOT_MARGIN - (DOT_SIZE / 2), DOT_MARGIN - (DOT_SIZE / 2),
							x + DOT_MARGIN - (DOT_SIZE / 2), y + DOT_MARGIN - (DOT_SIZE / 2)),
						Fill = Foreground,
					};
					gridClock.Children.Add(dot);
				}
			}
		}

		/// <summary>
		/// Sets the clock's time. The value of <value>TimeOffset</value> is added.
		/// </summary>
		protected void SetTime(DateTime d)
		{
			if (TimeOffset != 0)
				d += TimeSpan.FromSeconds(TimeOffset);

			double s = d.Second;
			double m = d.Minute;
			double h = d.Hour;
			double hm = h + (m / 60);    // Hour hand is adjusted for minutes

			handHoursAngle.Angle = hm * 30;    // = hm / 12 * 360;
			handMinutesAngle.Angle = m * 6;    // = m / 60 * 360;
			handSecondsAngle.Angle = s * 6;    // = s / 60 * 360;
		}

		#region Custom properties

		public static readonly DependencyProperty UseNumbersProperty =
			DependencyProperty.Register("UseNumbers", typeof(bool), typeof(AnalogClock));

		/// <summary>
		/// Gets or sets whether to show numbers or dots on the clock face.
		/// </summary>
		public bool UseNumbers
		{
			get { return (bool)GetValue(UseNumbersProperty); }
			set
			{
				/*
				if (value == UseNumbers)
					return;
				*/
				SetValue(UseNumbersProperty, value);
				OnPropertyChanged("UseNumbers");

				InitClockFace();
			}
		}

		public static readonly DependencyProperty TimeOffsetProperty =
			DependencyProperty.Register("TimeOffset", typeof(int), typeof(AnalogClock));

		/// <summary>
		/// Gets or sets an offset (in seconds) to add to the clock's time.
		/// </summary>
		public int TimeOffset
		{
			get { return (int)GetValue(TimeOffsetProperty); }
			set
			{
				/*
				if (value == TimeOffset)
					return;
				*/
				SetValue(TimeOffsetProperty, value);
				OnPropertyChanged("TimeOffset");

				SetTime(DateTime.Now);
			}
		}

		public static readonly DependencyProperty ClockFaceBrushProperty =
			DependencyProperty.Register("ClockFaceBrush", typeof(Brush), typeof(AnalogClock));

		/// <summary>
		/// Gets or sets an offset (in seconds) to add to the clock's time.
		/// </summary>
		[Category("Brushes")]
		public Brush ClockFaceBrush
		{
			get { return (Brush)GetValue(ClockFaceBrushProperty); }
			set
			{
				SetValue(ClockFaceBrushProperty, value);
				OnPropertyChanged("ClockFaceBrush");
			}
		}

		#endregion

		public event PropertyChangedEventHandler PropertyChanged;

		protected void OnPropertyChanged(string name)
		{
			PropertyChangedEventHandler handler = PropertyChanged;
			if (handler != null)
			{
				handler(this, new PropertyChangedEventArgs(name));
			}
		}

	}
}
